@@ -5,7 +5,7 @@ package DBIx::Log4perl::Constants;
 require Exporter;
 our @ISA = qw(Exporter);
 
-use constant DBIX_L4P_LOG_DEFAULT => 0x1d;
+use constant DBIX_L4P_LOG_DEFAULT => 0x5d;
 use constant DBIX_L4P_LOG_ALL => 0xffffff;
 use constant DBIX_L4P_LOG_INPUT => 0x1;
 use constant DBIX_L4P_LOG_OUTPUT => 0x2;
@@ -13,6 +13,7 @@ use constant DBIX_L4P_LOG_CONNECT => 0x4;
 use constant DBIX_L4P_LOG_TXN => 0x8;
 use constant DBIX_L4P_LOG_ERRCAPTURE => 0x10;
 use constant DBIX_L4P_LOG_WARNINGS => 0x20;
+use constant DBIX_L4P_LOG_ERRORS => 0x40;
 
 our $LogMask = DBIX_L4P_LOG_DEFAULT;
 
@@ -26,6 +27,7 @@ our @EXPORT_MASKS = qw(DBIX_L4P_LOG_DEFAULT
 		       DBIX_L4P_LOG_TXN
 		       DBIX_L4P_LOG_ERRCAPTURE
 		       DBIX_L4P_LOG_WARNINGS
+		       DBIX_L4P_LOG_ERRORS
 		     );
 our %EXPORT_TAGS = (masks => \@EXPORT_MASKS);
 Exporter::export_ok_tags('masks');
