@@ -1,4 +1,4 @@
-# $Id: st.pm 212 2006-06-23 16:44:43Z martin $
+# $Id: st.pm 265 2006-08-04 15:51:00Z martin $
 use strict;
 use warnings;
 use DBI;
@@ -8,23 +8,32 @@ package DBIx::Log4perl::st;
 @DBIx::Log4perl::st::ISA = qw(DBI::st DBIx::Log4perl);
 use DBIx::Log4perl::Constants qw (:masks $LogMask);
 
+sub finish {
+    my ($sth) = shift;
+    my $h = $sth->{private_DBIx_Log4perl};
+    
+    $sth->_dbix_l4p_debug('finish')
+	if ($h->{logmask} & DBIX_L4P_LOG_INPUT);
+    return $sth->SUPER::finish;
+}
+	    
 sub execute {
     my ($sth, @args) = @_;
     my $h = $sth->{private_DBIx_Log4perl};
 
     $sth->_dbix_l4p_debug('execute', @args)
-      if (($LogMask & DBIX_L4P_LOG_INPUT) && (caller !~ /^DBD::/));
+      if (($h->{logmask} & DBIX_L4P_LOG_INPUT) && (caller !~ /^DBD::/));
 
     my $ret = $sth->SUPER::execute(@args);
 
     if (!$ret) {		# error
 	$h->{logger}->error("\tfailed with " . $sth->errstr)
-	    if (($LogMask && DBIX_L4P_LOG_ERRCAPTURE) && # logging errors
+	    if (($h->{logmask} && DBIX_L4P_LOG_ERRCAPTURE) && # logging errors
 		(caller !~ /^DBD::/)); # not called from DBD e.g. execute_array
     } elsif (defined($ret)) {
         $sth->_dbix_l4p_debug('affected', $ret)
 	    if ((!defined($sth->{NUM_OF_FIELDS})) && # not a result-set
-		($LogMask & DBIX_L4P_LOG_INPUT)	&& # logging input
+		($h->{logmask} & DBIX_L4P_LOG_INPUT)	&& # logging input
 		(caller !~ /^DBD::/));
     }
     return $ret;
@@ -35,11 +44,20 @@ sub execute_array {
     my $h = $sth->{private_DBIx_Log4perl};
 
     $sth->_dbix_l4p_debug('execute_array', @args)
-      if ($LogMask & DBIX_L4P_LOG_INPUT);
+      if ($h->{logmask} & DBIX_L4P_LOG_INPUT);
+
+    if (($#args >= 0) && ($args[0]) &&
+	    (ref($args[0]) eq 'HASH') &&
+		(!exists($args[0]->{ArrayTupleStatus}))) {
+	$args[0]->{ArrayTupleStatus} = \my @tuple_status;
+    } elsif (!$args[0]) {
+	$args[0] = {ArrayTupleStatus => \my @tuple_status};
+    }
     my $executed = $sth->SUPER::execute_array(@args);
     if (!$executed) {
         #print Data::Dumper->Dump([$sth->{ParamArrays}], ['ParamArrays']), "\n";
-	return $executed if (ref($args[0] ne 'HASH') ||
+	return $executed unless ($h->{logmask} & DBIX_L4P_LOG_ERRORS);
+	return $executed if (!$args[0] || ref($args[0] ne 'HASH') ||
 			       !exists($args[0]->{ArrayTupleStatus}));
         my $pa = $sth->{ParamArrays};
         if (scalar(@args) > 0) {
@@ -50,7 +68,11 @@ sub execute_array {
 	    $sth->_dbix_l4p_error('Error', $ats->[$n]);
 	    my @plist;
 	    foreach my $p (keys %{$pa}) {
-	      push @plist, $pa->{$p}->[$n];
+		if (ref($pa->{$p})) {
+		    push @plist, $pa->{$p}->[$n];
+		} else {
+		    push @plist, $pa->{$p};
+		}
 	    }
 	    $h->{logger}->error(sub {"\t for " . join(',', @plist)});
 	  }
@@ -64,7 +86,7 @@ sub bind_param {
   my $h = $sth->{private_DBIx_Log4perl};
 
   $sth->_dbix_l4p_debug('bind_param', @args)
-    if ($LogMask & DBIX_L4P_LOG_INPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_INPUT);
   return $sth->SUPER::bind_param(@args);
 }
 
@@ -73,7 +95,7 @@ sub bind_param_inout {
   my $h = $sth->{private_DBIx_Log4perl};
 
   $sth->_dbix_l4p_debug('bind_param_inout', @args)
-    if ($LogMask & DBIX_L4P_LOG_INPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_INPUT);
   return $sth->SUPER::bind_param_inout(@args);
 }
 
@@ -82,7 +104,7 @@ sub bind_param_array {
   my $h = $sth->{private_DBIx_Log4perl};
 
   $sth->_dbix_l4p_debug('bind_param_array', @args)
-    if ($LogMask & DBIX_L4P_LOG_INPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_INPUT);
   return $sth->SUPER::bind_param_array(@args);
 }
 
@@ -92,7 +114,7 @@ sub fetch {			# alias for fetchrow_arrayref
 
   my $res = $sth->SUPER::fetch(@args);
   $h->{logger}->debug(sub {Data::Dumper->Dump([$res], ['fetch'])})
-    if ($LogMask & DBIX_L4P_LOG_OUTPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_OUTPUT);
   return $res;
 }
 
@@ -103,7 +125,7 @@ sub fetchrow_arrayref {			# alias for fetchrow_arrayref
   my $res = $sth->SUPER::fetchrow_arrayref(@args);
   $h->{logger}->debug(sub {Data::Dumper->Dump([$res],
 						   ['fetchrow_arrayref'])})
-    if ($LogMask & DBIX_L4P_LOG_OUTPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_OUTPUT);
   return $res;
 }
 
@@ -114,7 +136,7 @@ sub fetchrow_array {
   my @row = $sth->SUPER::fetchrow_array(@args);
   $h->{logger}->debug(sub {
 			     Data::Dumper->Dump([\@row], ['fetchrow_array'])})
-    if ($LogMask & DBIX_L4P_LOG_OUTPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_OUTPUT);
   return @row;
 }
 
@@ -125,7 +147,7 @@ sub fetchrow_hashref {
   my $res = $sth->SUPER::fetchrow_hashref(@args);
   $h->{logger}->debug(
       sub {Data::Dumper->Dump([$res], ['fetchrow_hashref'])})
-    if ($LogMask & DBIX_L4P_LOG_OUTPUT);
+    if ($h->{logmask} & DBIX_L4P_LOG_OUTPUT);
   return $res;
 }
 
